/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */

/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */
import java.io.File;
import java.util.*;

public class JMCADSystemProperties {

    public JMCADSystemProperties() {
        System.out.println("\n*** System Property ***");
        System.out.println("Number available processors: "+Runtime.getRuntime().availableProcessors());
        System.out.println("java.vendor: \t" + System.getProperty("java.vm.vendor"));
        System.out.println("java.runtime.name: \t" + System.getProperty("java.runtime.name"));
        System.out.println("java.version: \t" + System.getProperty("java.version"));
        System.out.println("Default Localization: \t" + Locale.getDefault());
        System.out.println("file.encoding: \t" + System.getProperty("file.encoding"));
        System.out.println("sun.jnu.encoding: \t" + System.getProperty("sun.jnu.encoding"));
        System.out.println("user.country: \t" + System.getProperty("user.country"));
        System.out.println("user.language: \t" + System.getProperty("user.language"));
        System.out.println("user.dir: \t" + System.getProperty("user.dir"));
        System.out.println("user.home: \t" + System.getProperty("user.home"));
        System.out.println("os.name: \t" + System.getProperty("os.name"));
        System.out.println("os.version: \t" + System.getProperty("os.version"));
        System.out.println("os.arch: \t" + System.getProperty("os.arch"));
        System.out.println("java.library.path: \t" + System.getProperty("java.library.path"));
        System.out.println("\n*** Set new System Property ***");
        String os_name = System.getProperty("os.name").toLowerCase();
        String os_version = System.getProperty("os.version").toLowerCase();
        String os_arch = System.getProperty("os.arch").toLowerCase();
        String java_library_path = System.getProperty("java.library.path");
        String java_library_path_add = "";
        if (os_name.indexOf("win") != -1) {
            if (os_arch.indexOf("64") != -1) {
                java_library_path_add = "lib\\win64";
            } else {
                java_library_path_add = "lib\\win32";
            }
        } else if (os_name.indexOf("nix") != -1 || os_name.indexOf("nux") != -1) {
            if (os_arch.indexOf("64") != -1) {
                java_library_path_add = "lib/x86_64-unknown-linux-gnu";
            } else {
                java_library_path_add = "lib/i686-pc-linux-gnu";
            }
        } else if (os_name.indexOf("mac") != -1) {
            if (os_version.indexOf("10.5") != -1) {
                java_library_path_add = "lib/mac-10.5";
            } else {
                java_library_path_add = "lib/mac-10.4";
            }
        } else if (os_name.indexOf("sun") != -1) {
            if (os_version.indexOf("2.10") != -1) {
                java_library_path_add = "lib/sparc-sun-solaris2.10";
            } else {
                java_library_path_add = "lib/sparc-sun-solaris2.6";
            }
        }
        if(java_library_path!=null && !java_library_path.contains(java_library_path_add)){
            java_library_path += File.pathSeparator + java_library_path_add;
            System.setProperty("java.library.path", java_library_path);
        }
        System.out.println("java.library.path: \t" + System.getProperty("java.library.path"));
        System.out.println();



    }
}
