/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */

/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */
import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import gnu.io.*;

public class _dev_RS232 extends JMCADObject implements SerialPortEventListener {

    private final Image img = new ImageIcon(this.getClass().getResource("_dev_RS232.gif")).getImage();
    private JComboBox cb_port = new JComboBox();
    private JComboBox cb_br = new JComboBox(new String[]{"110", "150", "300", "600", "1200", "2400", "4800", "9600", "19200", "38400", "57600", "115200"});
    private JComboBox cb_db = new javax.swing.JComboBox(new String[]{"5", "6", "7", "8"});
    private JComboBox cb_sb = new javax.swing.JComboBox(new String[]{"1", "1.5", "2"});
    private JComboBox cb_p = new javax.swing.JComboBox(new String[]{"NONE", "ODD", "EVEN", "MARK", "SPACE"});
    private String PortName = "";
    private int BaudRate = 9600;
    private int DataBits = 8;
    private int StopBits = 1;
    private String Parity = "NONE";
    private boolean PortChacked = false;
    private SerialPort SPort = null;
    private InputStream ins;
    private OutputStream outs;

    public void write(RandomAccessFile fout) throws IOException {
        super.write(fout);
        fout.writeBytes("PortName = " + PortName + "\n");
        fout.writeBytes("BaudRate = " + BaudRate + "\n");
        fout.writeBytes("DataBits = " + DataBits + "\n");
        fout.writeBytes("StopBits = " + StopBits + "\n");
        fout.writeBytes("Parity = " + Parity + "\n");
    }

    public void parse(String pst) {
        super.parse(pst);
        StringTokenizer stt = new StringTokenizer(pst, "\n");
        while (stt.hasMoreTokens()) {
            try {
                String st = stt.nextToken();
                st = st.trim();
                if (st.length() != 0) {
                    StringTokenizer stt0 = new StringTokenizer(st, " =,\t");
                    String st0 = stt0.nextToken();
                    if (st0.equalsIgnoreCase("PORTNAME")) {
                        try {
                            PortName = stt0.nextToken();
                        } catch (Exception e3) {
                            error(e3);
                        }
                    } else if (st0.equalsIgnoreCase("BAUNDRATE")) {
                        try {
                            BaudRate = Integer.parseInt(stt0.nextToken());
                        } catch (Exception e3) {
                            error(e3);
                        }
                    } else if (st0.equalsIgnoreCase("DATABITS")) {
                        try {
                            DataBits = Integer.parseInt(stt0.nextToken());
                        } catch (Exception e3) {
                            error(e3);
                        }
                    } else if (st0.equalsIgnoreCase("STOPBITS")) {
                        try {
                            StopBits = Integer.parseInt(stt0.nextToken());
                        } catch (Exception e3) {
                            error(e3);
                        }
                    } else if (st0.equalsIgnoreCase("PARITY")) {
                        try {
                            Parity = stt0.nextToken();
                        } catch (Exception e3) {
                            error(e3);
                        }
                    }
                }
            } catch (Exception e1) {
                error(e1);
            }
        }
        try {
            init();
        } catch (Exception e2) {
            error(e2);
        }
        JPanel pl = new JPanel();
        JPanel pr = new JPanel();
        pl.setBackground(Color.YELLOW);
        pr.setBackground(Color.GREEN);
        panel_c.setLayout(new GridLayout(1, 2));
        panel_c.add(pl);
        panel_c.add(pr);
    }

    public _dev_RS232() {
        // RS-232C Serial Port
        //    RS-232C    : 110, 150, 300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200 /.
        ToolTipText = "RS-232";
        in = new double[1];
        for (int i = 0; i < in.length; i++) {
            in[i] = Double.NaN;
        }
        in_text = new String[1];
        // TxD Transmit Data ( )
        in_text[0] = "TxD";
        // CTS Clear To Send ( )
        // in_text[1] = "CTS";
        // DSR Data Set Ready ( )
        // in_text[2] = "DSR";
        // DCD Carrie Detect ( )
        //in_text[3] = "DCD";
        // RI Ring Indicator ()
        //in_text[4] = "RI";
        out = new double[1];
        for (int i = 0; i < out.length; i++) {
            out[i] = Double.NaN;
        }
        out_text = new String[3];
        // RxD Receive Data ( )
        // out_text[0] = "RxD";
        // RTS Request To Send (  )
        // out_text[1] = "RTS";
        // DTR Data Terminal Ready ( )
        // out_text[2] = "DTR";
        out_obj = new Object[1];
        out_obj_index = new int[1];
        w = 76;
        h = 66;
        try {
            init();
        } catch (Exception e) {
            e.printStackTrace();
        }
        JPanel pl = new JPanel();
        JPanel pr = new JPanel();
        pl.setBackground(Color.YELLOW);
        pr.setBackground(Color.GREEN);
        panel_c.setLayout(new GridLayout(1, 2));
        panel_c.add(pl);
        panel_c.add(pr);
    }

    public void paint_info(Graphics g) {
        g.drawImage(img, 0, 15, this);
    }

    public void calc_pre() {
        try {
            CommPortIdentifier CPI = CommPortIdentifier.getPortIdentifier(PortName);
            SPort = (SerialPort) CPI.open(PortName, 2000);
            if (Parity.equalsIgnoreCase("NONE")) {
                SPort.setSerialPortParams(BaudRate, DataBits, StopBits, SerialPort.PARITY_NONE);
            } else if (Parity.equalsIgnoreCase("ODD")) {
                SPort.setSerialPortParams(BaudRate, DataBits, StopBits, SerialPort.PARITY_ODD);
            } else if (Parity.equalsIgnoreCase("EVEN")) {
                SPort.setSerialPortParams(BaudRate, DataBits, StopBits, SerialPort.PARITY_EVEN);
            } else if (Parity.equalsIgnoreCase("MARK")) {
                SPort.setSerialPortParams(BaudRate, DataBits, StopBits, SerialPort.PARITY_MARK);
            } else if (Parity.equalsIgnoreCase("SPACE")) {
                SPort.setSerialPortParams(BaudRate, DataBits, StopBits, SerialPort.PARITY_SPACE);
            }
            ins = SPort.getInputStream();
            outs = SPort.getOutputStream();
            SPort.removeEventListener();
            //System.out.println(SPort.isReceiveFramingEnabled() + " " + SPort.isReceiveThresholdEnabled() + " " + SPort.isReceiveTimeoutEnabled());
            //this.serialEvent(null);
            //System.out.println(SPort.getReceiveFramingByte());

            SPort.addEventListener(this);
            SPort.notifyOnDataAvailable(true);
            /*SPort.notifyOnBreakInterrupt(true);
            SPort.notifyOnCTS(true);
            SPort.notifyOnCarrierDetect(true);
            SPort.notifyOnDSR(true);
            SPort.notifyOnDataAvailable(true);
            SPort.notifyOnFramingError(true);
            SPort.notifyOnOutputEmpty(true);
            SPort.notifyOnOverrunError(true);
            SPort.notifyOnParityError(true);
            SPort.notifyOnRingIndicator(true);*/
        } catch (Exception e) {
            error(e);
        }
    }

    /*public void start(){
    
    }*/
    public void calc() {
        try {
            if (!Double.isInfinite(in[0])) {
                outs.write((int) in[0]);
                in[0] = Double.NaN;
            }
        } catch (Exception e) {
        }
    }

    public void calc_post() {
        try {
            ins.close();
            outs.close();
            SPort.close();
        } catch (Exception e) {
            error(e);
        }
    }

    public void edit_pre() {
        if (!PortChacked) {
            Vector v = new Vector();
            Enumeration ports = CommPortIdentifier.getPortIdentifiers();
            while (ports.hasMoreElements()) {
                CommPortIdentifier port = (CommPortIdentifier) ports.nextElement();
                if (port.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                    v.addElement(port.getName());
                }
            }
            String[] st_arr = new String[v.size()];
            for (int i = 0; i < st_arr.length; i++) {
                st_arr[i] = v.elementAt(i) + "";
            }
            cb_port.setModel(new DefaultComboBoxModel(st_arr));
            PortChacked = true;
        }
        cb_port.setSelectedItem(PortName);
        cb_br.setSelectedItem(BaudRate + "");
        cb_db.setSelectedItem(DataBits + "");
        cb_sb.setSelectedItem(StopBits + "");
        cb_p.setSelectedItem(Parity);
        GridBagConstraints gridBagConstraints;
        edit_panel = new JPanel(new GridBagLayout());
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Port") + ": "), gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(cb_port, gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Baud_Rate") + ": "), gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(cb_br, gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Data_Bits") + ": "), gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(cb_db, gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Stop_Bits") + ": "), gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(cb_sb, gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Parity") + ": "), gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(cb_p, gridBagConstraints);
    }

    public void edit_post() {
        try {
            PortName = cb_port.getSelectedItem() + "";
            BaudRate = Integer.parseInt(cb_br.getSelectedItem() + "");
            DataBits = Integer.parseInt(cb_db.getSelectedItem() + "");
            StopBits = Integer.parseInt(cb_sb.getSelectedItem() + "");
            Parity = cb_p.getSelectedItem() + "";
        } catch (Exception e) {
            error(e);
        }
    }

    public void serialEvent(SerialPortEvent event) {
        switch (event.getEventType()) {

            case SerialPortEvent.BI:

            case SerialPortEvent.OE:

            case SerialPortEvent.FE:

            case SerialPortEvent.PE:

            case SerialPortEvent.CD:

            case SerialPortEvent.CTS:

            case SerialPortEvent.DSR:

            case SerialPortEvent.RI:

            case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
                break;

            case SerialPortEvent.DATA_AVAILABLE:
                int data;
                try {
                    while ((data = ins.read()) > -1) {
                        if (!Double.isInfinite(data) && !Double.isNaN(data)) {
                            ((JMCADObject) out_obj[0]).set(data, out_obj_index[0]);
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
                break;
        }
    }
}
