/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */

/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */
import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.table.*;

import static org.jocl.CL.*;

import org.jocl.*;

public class _graph1_gpu extends JMCADObject {

    private BufferedImage offscreen;
    private BufferedImage offscreen_grid;
    private Graphics2D offgraphics;
    private Graphics2D offgraphics_grid;
    private Vector valueX;
    private Vector valueY;
    private float sx, sy, dx, dy, count, count_i;
    private float min_x, min_y, max_x, max_y;
    private int w_c, h_c;
    private boolean isPainted = false, isRt = false, isTable = false, isCuda = false;
    private JTextField tfw = new JTextField();
    private JTextField tfh = new JTextField();
    private JCheckBox cb = new JCheckBox(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Display_of_data_in_real_time"), false);
    private JCheckBox cb_table = new JCheckBox(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Display_of_data_in_the_form_of_the_table"), false);
    private TableModel tableModel;
    private String[] table_hdr;
    private JTable table;
    private boolean joclinit = false;
    /// JOCL VAR`S \\\
    private cl_kernel kernel;
    private cl_context context;
    private cl_program program;
    private cl_command_queue commandQueue;
    private static String programSource =
            "__kernel void "
            + "sampleKernel(__global float *X,"
            + "             __global float *Y,"
            + "             __global const float *param)"
            + "{ int index = get_global_id(0); "
            + "    X[index] = X[index]/param[0]+param[1];"
            + "    Y[index] = param[4] - Y[index]/param[2]-param[3];"
            + "}";
    /// JOCL VAR`S \\\

    public void write(RandomAccessFile fout) throws IOException {
        super.write(fout);
        fout.writeBytes("width = " + w_c + "\n");
        fout.writeBytes("height = " + h_c + "\n");
        fout.writeBytes("real_time = " + cb.isSelected() + "\n");
        fout.writeBytes("show_table = " + cb_table.isSelected() + "\n");
    }

    public void parse(String pst) {
        super.parse(pst);
        StringTokenizer stt = new StringTokenizer(pst, "\n");
        while (stt.hasMoreTokens()) {
            try {
                String st = stt.nextToken();
                st = st.trim();
                if (st.length() != 0) {
                    StringTokenizer stt0 = new StringTokenizer(st, " =,\t");
                    String st0 = stt0.nextToken();
                    if (st0.equalsIgnoreCase("WIDTH")) {
                        st0 = stt0.nextToken();
                        try {
                            w = Integer.parseInt(st0) + 40;
                        } catch (Exception e3) {
                            error(e3);
                        }
                        w_c = w - 40;
                    } else if (st0.equalsIgnoreCase("HEIGHT")) {
                        st0 = stt0.nextToken();
                        try {
                            h = Integer.parseInt(st0);
                        } catch (Exception e3) {
                            error(e3);
                        }
                        h_c = h;
                    } else if (st0.equalsIgnoreCase("REAL_TIME")) {
                        st0 = stt0.nextToken();
                        try {
                            isRt = Boolean.valueOf(st0).booleanValue();
                        } catch (Exception e3) {
                            error(e3);
                        }
                        cb.setSelected(isRt);
                    } else if (st0.equalsIgnoreCase("SHOW_TABLE")) {
                        st0 = stt0.nextToken();
                        try {
                            isTable = Boolean.valueOf(st0).booleanValue();
                        } catch (Exception e3) {
                            error(e3);
                        }
                        cb_table.setSelected(isTable);
                    }
                }
            } catch (Exception e1) {
                error(e1);
            }
        }
        try {
            init();
        } catch (Exception e2) {
            error(e2);
        }
    }

    public _graph1_gpu() {
        this.isVisual = true;
        isGeneranor = true;
        ToolTipText = ResourceBundle.getBundle("JMCAD_Internationalize").getString("The_time_graph:_Y_-_a_variable;_X_-_time_ti");
        in = new double[1];
        in[0] = Double.NaN;
        in_text = new String[1];
        in_text[0] = "IN";
        out = new double[1];
        out[0] = Double.NaN;
        out_text = new String[1];
        out_text[0] = "OUT";
        out_obj = new Object[1];
        out_obj_index = new int[1];
        w = 150;
        h = 100;
        w_c = 150;
        h_c = 100;
        try {
            init();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void paint_info(Graphics g) {
        isPainted = true;
        if (offscreen != null) {
            g.drawImage(offscreen, 0, 0, this);
        } else {
            g.drawImage(getImageGrid(), 0, 0, this);
        }
        isPainted = false;
    }

    private BufferedImage getImageGrid() {
        offscreen_grid = new BufferedImage(w_c, h_c, BufferedImage.TYPE_3BYTE_BGR);
        offgraphics_grid = (Graphics2D) offscreen_grid.getGraphics();
        //offgraphics_grid.clearRect(0,0,w_c,h_c);
        offgraphics_grid.setColor(this.panel_c.getBackground());
        offgraphics_grid.fillRect(0, 0, w_c, h_c);
        offgraphics_grid.setColor(Color.gray);
        offgraphics_grid.setStroke(new BasicStroke(1));
        double w_step = w_c / 10;
        for (double i = 0; i < w_c; i += w_step) {
            offgraphics_grid.draw(new Line2D.Double(i, 0, i, h_c));
        }
        double h_step = h_c / 10;
        for (double i = 0; i < h_c; i += h_step) {
            offgraphics_grid.draw(new Line2D.Double(0, i, w_c, i));
        }
        return offscreen_grid;
    }

    public void calc_pre() {
        long numBytes[] = new long[1];
        valueX = new Vector();
        valueY = new Vector();
        if (panel_c.getWidth() != 0) {
            w_c = panel_c.getWidth();
        }
        if (panel_c.getHeight() != 0) {
            h_c = panel_c.getHeight();
        }

        count_i = 0;
        count /= w_c;
        sx = 1;
        sy = 1;
        min_x = Float.MAX_VALUE;
        max_x = Float.MIN_VALUE;
        min_y = Float.MAX_VALUE;
        max_y = Float.MIN_VALUE;
        valueX.removeAllElements();
        valueY.removeAllElements();
        offscreen = (BufferedImage) panel_c.createImage(w_c, h_c);
        offgraphics = (Graphics2D) offscreen.getGraphics();

        ///////JOCL inti\\\\\\\
        try {
            // Obtain the platform IDs and initialize the context properties
            System.out.println("Obtaining platform...");
            cl_platform_id platforms[] = new cl_platform_id[1];
            clGetPlatformIDs(platforms.length, platforms, null);
            cl_context_properties contextProperties = new cl_context_properties();
            contextProperties.addProperty(CL_CONTEXT_PLATFORM, platforms[0]);

            // Create an OpenCL context on a GPU device
            context = clCreateContextFromType(
                    contextProperties, CL_DEVICE_TYPE_GPU, null, null, null);
            if (context == null) {
                // If no context for a GPU device could be created,
                // try to create one for a CPU device.
                context = clCreateContextFromType(
                        contextProperties, CL_DEVICE_TYPE_CPU, null, null, null);

                if (context == null) {
                    System.out.println("Unable to create a context");
                    return;
                } else System.out.println("Created an OpenCL context on a CPU device.");
                
            } else System.out.println("Created an OpenCL context on a GPU device.");

            // Enable exceptions and subsequently omit error checks in this sample
            CL.setExceptionsEnabled(true);

            // Get the list of GPU devices associated with the context
            clGetContextInfo(context, CL_CONTEXT_DEVICES, 0, null, numBytes);

            // Obtain the cl_device_id for the first device
            int numDevices = (int) numBytes[0] / Sizeof.cl_device_id;
            cl_device_id devices[] = new cl_device_id[numDevices];
            clGetContextInfo(context, CL_CONTEXT_DEVICES, numBytes[0],
                    Pointer.to(devices), null);

            // Create a command-queue
            commandQueue =
                    clCreateCommandQueue(context, devices[0], 0, null);

            program = clCreateProgramWithSource(context,
                    1, new String[]{programSource}, null, null);

            // Build the program
            clBuildProgram(program, 0, null, null, null, null);

            // Create the kernel
            kernel = clCreateKernel(program, "sampleKernel", null);
            joclinit = true;
        } catch (Exception e) {
            System.out.println("JOCL Error");
        }
        /////JOCL init\\\\\\\\\

        if (isTable) {
            if (valueX == null || valueY == null) {
                return;
            }
            panel_c.removeAll();
            table_hdr = new String[2];
            table_hdr[0] = "ti";
            table_hdr[1] = "Value";
            tableModel = new AbstractTableModel() {

                public int getColumnCount() {
                    return 2;
                }

                public int getRowCount() {
                    return valueX.size();
                }

                public Object getValueAt(int row, int col) {
                    if (col == 0) {
                        return valueX.elementAt(row);
                    } else {
                        return valueY.elementAt(row);
                    }
                }

                public String getColumnName(int col) {
                    return table_hdr[col];
                }

                public boolean isCellEditable(int row, int col) {
                    return false;
                }
            };
            table = new JTable(tableModel);
            JScrollPane sp = new JScrollPane();
            sp.getViewport().add(table, null);
            panel_c.setLayout(new BorderLayout());
            panel_c.add(sp, BorderLayout.CENTER);
            this.revalidate();
        }
    }

    public void calc() {
        out[0] = in[0];
        if (Double.isInfinite(in[0])) {
            return;
        }
        valueY.addElement(new Float(in[0]));
        min_y = (float) Math.min(min_y, in[0]);
        max_y = (float) Math.max(max_y, in[0]);
        sy = (float) ((max_y - min_y) / ((float) h_c));
        dy = (float) (-min_y / sy);
        in[0] = Double.NaN;
        if (!isRt) {
            return;
        }
        if (isTable && valueX != null && valueY != null) {
            table.revalidate();
            return;
        }
        if (count < count_i) {
            if (isPainted) {
                return;
            }
            offgraphics.drawImage(offscreen_grid, null, 0, 0);
            offgraphics.setColor(Color.BLUE);
            offgraphics.setStroke(new BasicStroke(2));
            //      
            float[] vx = new float[valueX.size()];
            float[] vy = new float[valueY.size()];
            //      valueX, valueY

            ///// JOCL CALC \\\\\\\
            if (joclinit) {
                for (int i = 0; i < valueX.size(); i++) {
                    vx[i] = ((Float) valueX.elementAt(i));
                }
                for (int i = 0; i < valueY.size(); i++) {
                    vy[i] = ((Float) valueY.elementAt(i));
                }
                float[] arr_params = new float[5];
                arr_params[0] = sx;
                arr_params[1] = dx;
                arr_params[2] = sy;
                arr_params[3] = dy;
                arr_params[4] = h_c;
                //pointers
                Pointer Pvx = Pointer.to(vx);
                Pointer Pvy = Pointer.to(vy);
                Pointer Parr_params = Pointer.to(arr_params);
                // Allocate the memory objects for the input- and output data
                cl_mem memObjects[] = new cl_mem[7];

                memObjects[0] = clCreateBuffer(context,
                        CL_MEM_READ_WRITE,
                        Sizeof.cl_float * vx.length, Pvx, null);
                memObjects[1] = clCreateBuffer(context,
                        CL_MEM_READ_WRITE,
                        Sizeof.cl_float * vy.length, Pvy, null);
                memObjects[2] = clCreateBuffer(context,
                        CL_MEM_READ_WRITE,
                        Sizeof.cl_float * arr_params.length, Parr_params, null);

                // Set the arguments for the kernel
                clSetKernelArg(kernel, 0,
                        Sizeof.cl_mem, Pointer.to(memObjects[0]));
                clSetKernelArg(kernel, 1,
                        Sizeof.cl_mem, Pointer.to(memObjects[1]));
                clSetKernelArg(kernel, 2,
                        Sizeof.cl_mem, Pointer.to(memObjects[2]));

                // Set the work-item dimensions
                long global_work_size[] = new long[]{(vx.length < vy.length) ? vy.length : vx.length};
                long local_work_size[] = new long[]{1};

                // Execute the kernel
                clEnqueueNDRangeKernel(commandQueue, kernel, 1, null,
                        global_work_size, local_work_size, 0, null, null);

                // Read the output data
                clEnqueueReadBuffer(commandQueue, memObjects[0], CL_TRUE, 0, vx.length * Sizeof.cl_float, Pvx, 0, null, null);
                clEnqueueReadBuffer(commandQueue, memObjects[1], CL_TRUE, 0, vy.length * Sizeof.cl_float, Pvy, 0, null, null);

                // Release kernel, program, and memory objects
                clReleaseMemObject(memObjects[0]);
                clReleaseMemObject(memObjects[1]);
                clReleaseMemObject(memObjects[2]);
            }
            ///// JOCL CALC \\\\\\\

            try {
                // 
                for (int i = 0; i < vx.length - 1; i++) {
                    offgraphics.draw(new Line2D.Double(vx[i], vy[i], vx[i + 1], vy[i + 1]));
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            repaint();
            count_i = 0;
        } else {
            count_i++;
        }
    }

    public void calc_post() {

        ///// JOCL \\\\\
        if (joclinit) {
            clReleaseKernel(kernel);
            clReleaseProgram(program);
            clReleaseCommandQueue(commandQueue);
            clReleaseContext(context);
        }
        ///// JOCL \\\\\

        if (isTable && valueX != null && valueY != null) {
            table.revalidate();
            return;
        }
        offgraphics.drawImage(offscreen_grid, null, 0, 0);
        offgraphics.setColor(Color.BLUE);
        offgraphics.setStroke(new BasicStroke(2));
        if (valueX == null || valueY == null) {
            return;
        }
        float[] vx = new float[valueX.size()];
        float[] vy = new float[valueY.size()];
        for (int i = 0; i < valueX.size() && i < valueY.size(); i++) {
            vx[i] = ((Float) valueX.elementAt(i)).floatValue() / sx + dx;
            vy[i] = h_c - ((Float) valueY.elementAt(i)).floatValue() / sy - dy;
        }
        try {
            for (int i = 0; i < vx.length - 1; i++) {
                offgraphics.draw(new Line2D.Double(vx[i], vy[i], vx[i + 1], vy[i + 1]));
            }
        } catch (Exception e) {
        }
        repaint();
    }

    public synchronized void start(long t0, long ti, long dt) {
        valueX.addElement(new Float(ti & 0xFFFFFF));
        min_x = (t0 & 0xFFFFFF);
        max_x = (ti & 0xFFFFFF);
        count = (t0 - ti) / h;
        sx = (float) ((max_x - min_x) / ((float) w_c));
        dx = (float) (-min_x / sx);
    }

    public void edit_pre() {
        edit_panel = new JPanel(new VerticalFlowLayout());
        tfw.setText("" + w_c);
        tfh.setText("" + h_c);
        cb.setSelected(isRt);
        cb_table.setSelected(isTable);
        JPanel p0 = new JPanel(new BorderLayout());
        JPanel p1 = new JPanel(new BorderLayout());
        p0.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Width")), BorderLayout.WEST);
        p0.add(tfw, BorderLayout.CENTER);
        p1.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Height")), BorderLayout.WEST);
        p1.add(tfh, BorderLayout.CENTER);
        edit_panel.add(p0);
        edit_panel.add(p1);
        edit_panel.add(cb);
        edit_panel.add(cb_table);
    }

    public void edit_post() {
        try {
            setSize(Integer.parseInt(tfw.getText()) + 40, Integer.parseInt(tfh.getText()));
            validate();
            revalidate();
            w_c = panel_c.getWidth();
            h_c = panel_c.getHeight();
            size_visual_w = w_c;
            size_visual_h = h_c;
            sx = (float) (max_x - min_x) / w_c;
            dx = (float) -min_x / sx;
            sy = (float) (max_y - min_y) / h_c;
            dy = (float) -min_y / sy;
            offscreen = (BufferedImage) panel_c.createImage(w_c, h_c);
            offgraphics = (Graphics2D) offscreen.getGraphics();
            isRt = false;
            calc_post();
            panel_c.removeAll();
            isRt = cb.isSelected();
            isTable = cb_table.isSelected();
            if (isTable) {
                if (valueX == null || valueY == null) {
                    return;
                }
                panel_c.removeAll();
                table_hdr = new String[2];
                table_hdr[0] = "ti";
                table_hdr[1] = "Value";
                tableModel = new AbstractTableModel() {

                    public int getColumnCount() {
                        return 2;
                    }

                    public int getRowCount() {
                        return valueY.size();
                    }

                    public Object getValueAt(int row, int col) {
                        if (col == 0) {
                            return valueX.elementAt(row);
                        } else {
                            return valueY.elementAt(row);
                        }
                    }

                    public String getColumnName(int col) {
                        return table_hdr[col];
                    }

                    public boolean isCellEditable(int row, int col) {
                        return false;
                    }
                };
                table = new JTable(tableModel);
                JScrollPane sp = new JScrollPane();
                sp.getViewport().add(table, null);
                panel_c.setLayout(new BorderLayout());
                panel_c.add(sp, BorderLayout.CENTER);
            }
            validate();
        } catch (Exception e) {
            error(e);
        }


    }
}
