/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */

/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */
import javax.swing.*;
import java.util.*;
import javax.sound.sampled.*;
import java.awt.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class _soundout extends JMCADObject {

    private final Image img = new ImageIcon(this.getClass().
            getResource("_soundout.gif")).getImage();
    private float sampleRate = 44100.0F;   //8000,11025,16000,22050,44100
    private int sampleSizeInBits = 16;    //8,16
    private int channels = 1;            //1,2
    private boolean signed = true;      //true,false
    private boolean bigEndian = false; //true,false
    private SourceDataLine sourceDataLine;
    private AudioFormat audioFormat;
    private byte[] b_one_channel = new byte[2];
    private byte[] b_two_channel = new byte[4];
    private JComboBox cb_sampleRate = new JComboBox(new String[]{"8000.0F",
                "11025.0F", "16000.0F", "22050.0F", "44100.0F"});
    private JComboBox cb_channels = new JComboBox(new String[]{"1", "2"});
    private JComboBox cb_bigEndian =
            new JComboBox(new String[]{"true", "false"});

    public void edit_pre() {
        cb_sampleRate.setSelectedItem(sampleRate + "");
        cb_channels.setSelectedItem(channels + "");
        cb_bigEndian.setSelectedItem(bigEndian + "");
        GridBagConstraints gridBagConstraints;
        edit_panel = new JPanel(new GridBagLayout());
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Sample_rate") + ": "),
                gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(cb_sampleRate, gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("Channels") + ": "),
                gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(cb_channels, gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(new JLabel("Big Endian: "), gridBagConstraints);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        edit_panel.add(cb_bigEndian, gridBagConstraints);
    }

    public void edit_post() {
        try {
            sampleRate = Float.parseFloat(cb_sampleRate.getSelectedItem() + "");
            channels = Integer.parseInt(cb_channels.getSelectedItem() + "");
            bigEndian = Boolean.parseBoolean(cb_bigEndian.getSelectedItem() + "");
        } catch (Exception e) {
            error(e);
        }
    }

    public _soundout() {
        ToolTipText = ResourceBundle.getBundle("JMCAD_Internationalize").getString("Exit_from_the_document_on_the_device") + " SOUND OUT = IN";
        in = new double[1];
        in_text = new String[1];
        in[0] = Double.NaN;
        in_text[0] = "IN";
        out = new double[0];
        out_text = new String[0];
        out_obj = new Object[0];
        out_obj_index = new int[0];
        w = 66;
        h = 36;
        this.panel_c.setBackground(Color.YELLOW);
        try {
            init();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void paint_info(Graphics g) {
        g.drawImage(img, 0, 0, this);
    }

    public void calc_pre() {
        try {
            audioFormat = new AudioFormat(sampleRate, sampleSizeInBits, channels, signed, bigEndian);
            DataLine.Info dataLineInfo = new DataLine.Info(SourceDataLine.class, audioFormat);
            sourceDataLine = (SourceDataLine) AudioSystem.getLine(dataLineInfo);
            sourceDataLine.open(audioFormat);
            sourceDataLine.start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public synchronized void calc() {
        int i = (int) (in[0] > 65535 ? 65535 : in[0]);
        if (bigEndian == false) {
            if (channels == 1) {
                b_one_channel[0] = (byte) (i & 0xFF);;
                b_one_channel[1] = (byte) ((i >> 8) & 0xFF);
                sourceDataLine.write(b_one_channel, 0, 2);
            } else {
                b_two_channel[0] = (byte) (i & 0xFF);;
                b_two_channel[1] = (byte) ((i >> 8) & 0xFF);
                b_two_channel[2] = (byte) (i & 0xFF);;
                b_two_channel[3] = (byte) ((i >> 8) & 0xFF);
                sourceDataLine.write(b_two_channel, 0, 4);
            }
        } else {
            if (channels == 1) {
                b_one_channel[0] = (byte) ((i >> 8) & 0xFF);
                b_one_channel[1] = (byte) (i & 0xFF);;
                sourceDataLine.write(b_one_channel, 0, 2);
            } else {
                b_two_channel[0] = (byte) ((i >> 8) & 0xFF);
                b_two_channel[1] = (byte) (i & 0xFF);;
                b_two_channel[2] = (byte) ((i >> 8) & 0xFF);
                b_two_channel[3] = (byte) (i & 0xFF);;
                sourceDataLine.write(b_two_channel, 0, 4);
            }
        }
        in[0] = Double.NaN;
    }

    public void calc_post() {
        sourceDataLine.drain();
        sourceDataLine.stop();
        sourceDataLine.close();
    }
}