/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOUSE. See the GNU
 * General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public License
 * along with this program; if not write to the Free Software
 * Foundation, inc., 59 Temple Place, Suite 330, Boston MA 02111-1307
 * USA
 */


/**
 *
 * @author Yuriy Mikhaylovskiy
 * @email YuriyMikhaylovskiy@yahoo.com
 * 
 */

import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.io.*;

public class _trapezoidal_wave extends JMCADObject {

    private final Image img = new ImageIcon(this.getClass().getResource("_trapezoidal_wave.gif")).getImage();
    JTextField tf_a = new JTextField();
    JTextField tf_b = new JTextField();
    JTextField tf_c = new JTextField();
    JTextField tf_d = new JTextField();
    JTextField tf_v = new JTextField();
    double a = 100, b = 200, c = 200, d = 1000, v = 500;

    public void write(RandomAccessFile fout) throws IOException {
        super.write(fout);
        fout.writeBytes("a = " + a + "\n");
        fout.writeBytes("b = " + b + "\n");
        fout.writeBytes("c = " + c + "\n");
        fout.writeBytes("d = " + d + "\n");
        fout.writeBytes("e = " + v + "\n");
    }

    public void parse(String pst) {
        super.parse(pst);
        StringTokenizer stt = new StringTokenizer(pst, "\n");
        while (stt.hasMoreTokens()) {
            try {
                String st = stt.nextToken();
                st = st.trim();
                if (st.length() != 0) {
                    StringTokenizer stt0 = new StringTokenizer(st, " =,\t");
                    String st0 = stt0.nextToken();
                    if (st0.equalsIgnoreCase("A")) {
                        st0 = stt0.nextToken();
                        if (!st0.equalsIgnoreCase("NAN")) {
                            try {
                                a = Double.parseDouble(st0);
                            } catch (Exception e3) {
                                error(e3);
                            }
                        }
                    } else if (st0.equalsIgnoreCase("B")) {
                        st0 = stt0.nextToken();
                        if (!st0.equalsIgnoreCase("NAN")) {
                            try {
                                b = Double.parseDouble(st0);
                            } catch (Exception e3) {
                                error(e3);
                            }
                        }
                    } else if (st0.equalsIgnoreCase("C")) {
                        st0 = stt0.nextToken();
                        if (!st0.equalsIgnoreCase("NAN")) {
                            try {
                                c = Double.parseDouble(st0);
                            } catch (Exception e3) {
                                error(e3);
                            }
                        }
                    } else if (st0.equalsIgnoreCase("D")) {
                        st0 = stt0.nextToken();
                        if (!st0.equalsIgnoreCase("NAN")) {
                            try {
                                d = Double.parseDouble(st0);
                            } catch (Exception e3) {
                                error(e3);
                            }
                        }
                    } else if (st0.equalsIgnoreCase("E")) {
                        st0 = stt0.nextToken();
                        if (!st0.equalsIgnoreCase("NAN")) {
                            try {
                                v = Double.parseDouble(st0);
                            } catch (Exception e3) {
                                error(e3);
                            }
                        }
                    }
                }
            } catch (Exception e1) {
                error(e1);
            }
        }
        try {
            init();
        } catch (Exception e2) {
            error(e2);
        }
    }

    public _trapezoidal_wave() {
        isGeneranor = true;
        ToolTipText = ResourceBundle.getBundle("JMCAD_Internationalize").getString("Trapezoidal_wave");
        in = new double[0];
        in_text = new String[0];
        out = new double[1];
        out[0] = Double.NaN;
        out_text = new String[1];
        out_text[0] = "OUT";
        out_obj = new Object[1];
        out_obj_index = new int[1];
        w = 70;
        h = 36;
        this.panel_c.setBackground(Color.GREEN);
        try {
            init();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void edit_pre() {
        edit_panel = new JPanel(new BorderLayout());
        edit_panel.add(new JLabel(new ImageIcon(this.getClass().getResource("_trapezoidal_wave_cfg.gif"))), BorderLayout.CENTER);
        JPanel p = new JPanel(new GridLayout(5, 1));
        JPanel p1 = new JPanel(new BorderLayout());
        JPanel p2 = new JPanel(new BorderLayout());
        JPanel p3 = new JPanel(new BorderLayout());
        JPanel p4 = new JPanel(new BorderLayout());
        JPanel p5 = new JPanel(new BorderLayout());
        tf_a.setText(a + "");
        tf_b.setText(b + "");
        tf_c.setText(c + "");
        tf_d.setText(d + "");
        tf_v.setText(v + "");
        p1.add(new JLabel("a = "), BorderLayout.WEST);
        p1.add(tf_a, BorderLayout.CENTER);
        p1.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("double")), BorderLayout.EAST);
        p2.add(new JLabel("b = "), BorderLayout.WEST);
        p2.add(tf_b, BorderLayout.CENTER);
        p2.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("ms")), BorderLayout.EAST);
        p3.add(new JLabel("c = "), BorderLayout.WEST);
        p3.add(tf_c, BorderLayout.CENTER);
        p3.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("ms")), BorderLayout.EAST);
        p4.add(new JLabel("d = "), BorderLayout.WEST);
        p4.add(tf_d, BorderLayout.CENTER);
        p4.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("ms")), BorderLayout.EAST);
        p5.add(new JLabel("v = "), BorderLayout.WEST);
        p5.add(tf_v, BorderLayout.CENTER);
        p5.add(new JLabel(ResourceBundle.getBundle("JMCAD_Internationalize").getString("ms")), BorderLayout.EAST);
        edit_panel.add(p, BorderLayout.SOUTH);
        p.add(p1);
        p.add(p2);
        p.add(p3);
        p.add(p4);
        p.add(p5);
    }

    public void edit_post() {
        try {
            double ai = Double.parseDouble(tf_a.getText());
            double bi = Double.parseDouble(tf_b.getText());
            double ci = Double.parseDouble(tf_c.getText());
            double di = Double.parseDouble(tf_d.getText());
            double vi = Double.parseDouble(tf_v.getText());
            a = ai;
            b = bi;
            c = ci;
            d = di;
            v = vi;
        } catch (Exception e) {
            error(e);
        }
    }

    public void paint_info(Graphics g) {
        g.drawImage(img, panel_c.getWidth() / 2 - 18, panel_c.getHeight() / 2 - 18, this);
    }

    public synchronized void start(long t0, long ti, long dt) {
        double dti = ti - t0 - dt;
        if (dti > (d + v)) {
            dti = dti % (d + v);
        }
        if (dti <= b) {
            out[0] = a * dti / b;
        } else if ((dti > b) && (dti <= (d - c))) {
            out[0] = a;
        } else if ((dti > (d - c)) && (dti <= d)) {
            out[0] = a * (1 - ((dti - d + c) / c));
        } else if ((dti > d) && (dti <= d + v)) {
            out[0] = 0;
        }
        put();
    }
}
